const { exec } = require('child_process');
const glob = require('glob');
const colors = require('colors');

function ExternalCompilerPlugin(params) {
  this.external = params.compiler;
  this.params = params.params || [];
  this.path = params.path;
  this.oldFileTimestamps = new Map();
}

ExternalCompilerPlugin.prototype.apply = function(compiler) {
  compiler.hooks.emit.tapAsync('ExternalCompilerPlugin', function(param, callback){
    glob(this.path, function(er, files) {
      var changed = false;
      files.forEach(function(file){
        param.fileDependencies.add(file);
        var oldTimestamp = this.oldFileTimestamps.get(file);
        var newTimestamp = param.fileTimestamps.get(file);
        if(!oldTimestamp && !newTimestamp){
          changed = true;
        } else if(!oldTimestamp || !newTimestamp){
        } else if(!(oldTimestamp === newTimestamp)){
          changed = true;
        }
      }.bind(this));
      this.oldFileTimestamps = new Map(param.fileTimestamps);
      if(changed){
        console.log('Starting ' + colors.green(this.external) + ' compiler...');
        console.log('');
        exec(this.external, function(error, stdout, stderr){
          console.log(stdout);
          console.log(colors.red(stderr));
          console.log('Finished!');
          callback();
        }.bind(this));
      }else{
        console.log('Nothing changed!');
        callback();
      }
    }.bind(this));
  }.bind(this));
};

module.exports = ExternalCompilerPlugin;
